#!/usr/bin/env python

# This file is part of A Robot's Conundrum.
#
# A Robot's Conundrum is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# A Robot's Conundrum is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# A Robot's Conundrum.  If not, see <http://www.gnu.org/licenses/>.
#
# ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '
#
#                         direction.py
#                     --------------------
#       date created : Tue Aug 7 2012
#          copyright : (C) 2012 Niels G. W. Serup
#      maintained by : Niels G. W. Serup <ngws@metanohi.name>

"""Directions."""

class Direction(object):
    @staticmethod
    def next_pos(pos):
        raise NotImplementedError

    @staticmethod
    def to_str():
        raise NotImplementedError

    @staticmethod
    def from_sakse(p):
        return {(0, -1): Up,
                (0, 1): Down,
                (-1, 0): Left,
                (1, 0): Right}[p]

class Up(Direction):
    @staticmethod
    def next_pos(pos):
        x, y = pos
        return x, y - 1

    @staticmethod
    def to_str():
        return 'up'

class Right(Direction):
    @staticmethod
    def next_pos(pos):
        x, y = pos
        return x + 1, y

    @staticmethod
    def to_str():
        return 'right'

class Down(Direction):
    @staticmethod
    def next_pos(pos):
        x, y = pos
        return x, y + 1

    @staticmethod
    def to_str():
        return 'down'

class Left(Direction):
    @staticmethod
    def next_pos(pos):
        x, y = pos
        return x - 1, y

    @staticmethod
    def to_str():
        return 'left'

all_directions = [Up, Right, Down, Left]
_sp = lambda n: lambda d: all_directions[(all_directions.index(d) + n) % 4]
succ, pred = _sp(1), _sp(-1)
isDirection = all_directions.__contains__
